#include <windows.h>
#include <stdlib.h>
#include <stdio.h>
#include <time.h>
#include "resource.h"
#include "plugin.h"


static HINSTANCE hInstance = NULL;
static POINT position = {0, 0};

typedef struct PREFS {
  unsigned int enabled;
  unsigned int lvls;
  unsigned int files;
  char wordsFile[MAX_PATH];
  int wordsLines;
} PREFS;


BOOL APIENTRY DllMain( HANDLE hModule, DWORD reason, LPVOID resvd )
{
  hInstance = (HINSTANCE)hModule;
  return TRUE;
}


static void CenterDialog( HWND hwndDlg )
{
  HWND hwndOwner; 
  RECT rc, rcDlg, rcOwner; 

  hwndOwner = GetParent(hwndDlg);
  if (hwndOwner == NULL) hwndOwner = GetDesktopWindow(); 
  GetWindowRect(hwndOwner, &rcOwner); 
  GetWindowRect(hwndDlg, &rcDlg); 
  CopyRect(&rc, &rcOwner); 
  OffsetRect(&rcDlg, -rcDlg.left, -rcDlg.top); 
  OffsetRect(&rc, -rc.left, -rc.top); 
  OffsetRect(&rc, -rcDlg.right, -rcDlg.bottom); 
  SetWindowPos(hwndDlg, HWND_TOP, rcOwner.left + (rc.right / 2),
	       rcOwner.top + (rc.bottom / 2), 0, 0, SWP_NOSIZE); 
}

static void PositionWindow( HWND hdlg, POINT *pt)
{
  HDC hdc;
  if (pt->x!=0 && pt->y!=0) {
    hdc=CreateIC("DISPLAY", "", "", NULL);
    pt->x = min( GetDeviceCaps(hdc, HORZRES)-100, pt->x);
    pt->x = max( pt->x, 0 );
    pt->y = min( GetDeviceCaps(hdc, VERTRES)-100, pt->y);
    pt->y = max( pt->y, 0 );
    DeleteDC(hdc);
    SetWindowPos(hdlg, HWND_TOP, pt->x, pt->y, 0,0, SWP_NOSIZE);
  } else
    CenterDialog(hdlg);
}

static void SaveWindowPosition( HWND hdlg, POINT *pt )
{
  RECT saverect;
  GetWindowRect(hdlg, &saverect);
  pt->x = saverect.left;
  pt->y = saverect.top;
}

EXPORT int FileFilterInitialize( MINIFILEPARAM *param )
{
  DWORD type, len;
  PREFS *prefs;

  // First, check that we can grok this plugin version!
  if (!param) return FALSE;
  if (param->version != PLUGINVERSION) return FALSE;

  strncpy(param->title, "Random File Generator", MAXTITLE);
  param->version = PLUGINVERSION;
  param->user = calloc(sizeof(PREFS), 1);
  prefs = (PREFS*) param->user;
  prefs->enabled = FALSE;
  prefs->lvls = 2;
  prefs->files = 100;
  prefs->wordsFile[0] = 0;
  prefs->wordsLines = 0;

  len = sizeof(PREFS);
  type = REG_BINARY;
  RegQueryValueEx(param->userKey, "prefs", 0, &type,
		  (unsigned char *)param->user, &len);

  type = REG_BINARY;
  len = sizeof(POINT);
  RegQueryValueEx(param->userKey, "position", 0, &type,
		  (unsigned char *)&position, &len);

  return TRUE;
}



static BOOL CALLBACK Dialog(HWND hDlg, UINT msg, WPARAM wParam, LPARAM lParam)
{
  static MINIFILEPARAM *param;
  static PREFS *prefs;
  int enabled;
  OPENFILENAME ofn;

  switch(msg) {
  case WM_USER:
    enabled = (SendDlgItemMessage(hDlg, IDC_ENABLE, BM_GETCHECK, 0, 0)==
	       BST_CHECKED)?TRUE:FALSE;
    EnableWindow(GetDlgItem(hDlg, IDC_STATIC1), enabled);
    EnableWindow(GetDlgItem(hDlg, IDC_STATIC2), enabled);
    EnableWindow(GetDlgItem(hDlg, IDC_LEVELS), enabled);
    EnableWindow(GetDlgItem(hDlg, IDC_FILES), enabled);
    EnableWindow(GetDlgItem(hDlg, IDC_WORDS), enabled);
    return TRUE;

  case WM_INITDIALOG:
    param = (MINIFILEPARAM*)lParam;
    prefs = (PREFS*)param->user;
    PositionWindow(hDlg, &position);
    SendDlgItemMessage(hDlg, IDC_ENABLE, BM_SETCHECK,
		       prefs->enabled?BST_CHECKED:BST_UNCHECKED, 0);
    SetDlgItemInt(hDlg, IDC_LEVELS, prefs->lvls, FALSE);
    SetDlgItemInt(hDlg, IDC_FILES, prefs->files, FALSE);
    SetFocus(GetDlgItem(hDlg, IDC_ENABLE));
    SendMessage(hDlg, WM_USER, 0, 0);
    return FALSE;


  case WM_COMMAND:
    switch (LOWORD(wParam)) {
    case IDOK:
      SaveWindowPosition(hDlg, &position);
      prefs->enabled = SendDlgItemMessage(hDlg, IDC_ENABLE, BM_GETCHECK, 0, 0)
	==BST_CHECKED?TRUE:FALSE;
      prefs->lvls = GetDlgItemInt(hDlg, IDC_LEVELS, NULL, FALSE);
      prefs->files = GetDlgItemInt(hDlg, IDC_FILES, NULL, FALSE);
      EndDialog(hDlg, TRUE);
      return TRUE;
		
    case IDCANCEL:
      SaveWindowPosition(hDlg, &position);
      EndDialog(hDlg, FALSE);
      return TRUE;

    case IDC_ENABLE:
      return SendMessage(hDlg, WM_USER, 0, 0);
		
    case IDC_WORDS:
      memset(&ofn, 0, sizeof(OPENFILENAME));
      ofn.lStructSize = sizeof(OPENFILENAME);
      ofn.hwndOwner = hDlg;
      ofn.lpstrTitle = "Choose Dictionary File...";
      ofn.lpstrFile = prefs->wordsFile;
      ofn.nMaxFile = MAX_PATH;
      ofn.lpstrFilter = "All Files (*.*)\0*.*\0\0";
      ofn.nFilterIndex = 1;
      ofn.lpstrFileTitle = NULL;
      ofn.nMaxFileTitle = 0;
      ofn.lpstrInitialDir = ".";
      ofn.Flags = OFN_PATHMUSTEXIST | OFN_HIDEREADONLY | OFN_EXPLORER;
      if (GetOpenFileName(&ofn)) {
	prefs->wordsLines = 0;
      }
      return TRUE;


    }
    break;
  }
  return FALSE;
}



EXPORT void FileFilterUserInterface( MINIFILEPARAM *param, HWND parent )
{
  DialogBoxParam(hInstance, MAKEINTRESOURCE(IDD_DIALOG), parent,
		 Dialog, (LPARAM)param);
}

static unsigned int lrand()
{
  unsigned int a, b, c;
  a = rand();
  b = rand();
  c = rand();
  a |= b<<15;
  a |= c<<30;
  return a;
}

static char *RandName(PREFS *prefs, int maxlen)
{
  FILE *fp;
  static int read = 0;
  static char buff[512];
  static char **word;
  char *allowed="qwertyuiopasdfghjklzxcvbnm1234567890";
  int i, j;

  if (!prefs->wordsLines && prefs->wordsFile[0]) {
    if (word) free(word);
    word = NULL;
    fp = fopen(prefs->wordsFile, "r");
    if (fp) {
      do {
	if (fgets(buff, 512, fp)) prefs->wordsLines++;
      } while (!feof(fp));
      fclose(fp);
    }
  }

  if (prefs->wordsLines && !word) {
    fp = fopen(prefs->wordsFile, "r");
    if (fp) {
      word = (char **)calloc(prefs->wordsLines, sizeof(char*));
      for (i=0; i<prefs->wordsLines; i++) {
	fgets(buff, 512, fp);
	buff[strlen(buff)-1] = 0;
	word[i] = (char *)malloc(strlen(buff)+1);
	strcpy(word[i], buff);
      }
      fclose(fp);
    }
  }

  if (word) {
    buff[0] = 0;
    do {
      strcat(buff, word[lrand()%prefs->wordsLines]);
      strcat(buff, " ");
    } while ((int)strlen(buff)<(int)maxlen && (rand()%2)==1);
    buff[maxlen-1] = 0;
  } else {
    j = rand()%maxlen;
    buff[0] = 0;
    for (i=0; i<j; i++)
      buff[i]=((rand()%6)==1)?' ':allowed[rand()%strlen(allowed)];
    buff[i] = 0;
  }
  buff[0] = toupper(buff[0]);
  return buff;
}

MINIZIPINFO *CreateRandom( MINIFILEPARAM *param, PREFS *prefs, int level )
{
  int files;
  int i;
  MINIZIPINFO *mz;
  char buff[32];

  files = rand()%prefs->files;
  mz=param->CreateMiniZipInfo(files);
  for (i=0; i<files; i++) {
    strncpy(mz->fi[i].name, RandName(prefs, NAMELEN), NAMELEN);
    itoa(rand()%300, buff, 10);
    strncpy(mz->fi[i].comm, buff, COMMLEN);
    strncat(mz->fi[i].comm, "KB", COMMLEN);
    mz->fi[i].size = rand();
    mz->fi[i].isdir = FALSE;
    mz->fi[i].showdir = FALSE;
    mz->fi[i].format = 0;
    mz->fi[i].zi = NULL;
    if ((rand()%20)==1 && level<(int)prefs->lvls) {
      // Make it a dir!
      strncpy(mz->fi[i].comm, "[dir]", COMMLEN);
      mz->fi[i].isdir = TRUE;
      mz->fi[i].showdir = TRUE;
      mz->fi[i].zi = CreateRandom(param, prefs, level+1);
    }

  }
  return mz;
}


EXPORT MINIZIPINFO *FileFilterExecute( MINIFILEPARAM *param,
				       MINIZIPINFO *mz,
				       char *label,
				       unsigned int labelSize,
				       char *path )
{
  PREFS *prefs;

  prefs = (PREFS*)param->user;

  if (!prefs->enabled) return mz;
	
  param->FreeMiniZipInfo(mz);

  srand((unsigned)time( NULL ));
  mz = CreateRandom( param, prefs, 0 );

  strncpy(label, RandName( prefs, labelSize ), labelSize );
  label[labelSize-1] = 0;

  return mz;
}

EXPORT void FileFilterUninitialize( MINIFILEPARAM *param )
{
  RegSetValueEx(param->userKey, "position", 0, REG_BINARY,
		(unsigned char*)&position, sizeof(POINT));
  RegSetValueEx(param->userKey, "prefs", 0, REG_BINARY,
		(unsigned char*)param->user, sizeof(PREFS));
  free(param->user);
}


